/*
L'interpréteur pixlib prend en argument le chemin du script à interpréter.
L'interpréteur retourne 1 si erreur, sinon 0.

Compilation
-----------
POUR COMPILER UN PROGRAMME TCC
	$ gcc -s -o prog prog.c -I./libtcc -L./libtcc -ltcc -ldl
	ou
	$ gcc -s -o prog prog.c libtcc/libtcc.a -ldl

POUR COMPILER UN EXE QUI UTILISE PIXLIB.SO
	$ export LD_LIBRARY_PATH=/home/masta/pixlib2021/libpixlib:$LD_LIBRARY_PATH
	$ gcc -Wall -s -o degrade degrade.c -I./libpixlib -L./libpixlib -lpixlib
	ou
	$ gcc -Wall -s -o degrade degrade.c libpixlib/libpixlib.so

Pour que l'exe trouve la librairie sans variable d'environnement:
	$ gedit /etc/ld.so.conf (dedans: include /etc/ld.so.conf.d/ *.conf) donc
	$ sudo gedit /etc/ld.so.conf.d/pixlib.conf
	Coller le chemin de pixlib.so /home/masta/pixlib/libpixlib et enregistrer.

DONC POUR COMPILER L'INTERPRETEUR
	$ gcc -Wall -s -o pixlib interpreteur.c -I./libpixlib -L./libpixlib -lpixlib -I./libtcc -L./libtcc -ltcc -ldl

Eécuter l'interpréteur
----------------------
	$ ./pixlib scripts/test.c
*/
#include <stdlib.h>
#include <stdio.h>
#include <pixlib.h>
#include <libtcc.h>
//==============================================================================
char* file_get_contents(const char* file)
{
	FILE* f = fopen (file, "rb");
	if (f==NULL) return NULL;

	fseek (f, 0, SEEK_END);
	long len = ftell (f);
	rewind (f);

	char* s = (char*) malloc (len);
	if (s == NULL) return NULL;

	fread (s,1,len,f);
	fclose (f);
	return s;
}
//==============================================================================
int main(int argc, char **argv)
{
	if(!argv[1]){
		puts("usage: ./pixlib script.c");
		return 1;
	}

	TCCState *tcc = tcc_new();
	if (!tcc) {
		fprintf(stderr, "Could not create tcc state\n");
		return 1;
	}

	tcc_set_lib_path(tcc, "./libtcc"); // pour lancer exécution sans passer d'argument
	tcc_add_include_path(tcc, "./libtcc");
	tcc_set_lib_path(tcc, "./libpixlib");
	tcc_add_include_path(tcc, "./libpixlib");
	tcc_set_output_type(tcc, TCC_OUTPUT_MEMORY);

	char* script = file_get_contents(argv[1]);
	if(!script) {
		printf("can't find or empty file [%s]\n", argv[1]);
		return 1;
	}
	tcc_compile_string(tcc, script);
	free(script);

	// FONCTIONS EXPOSÉES

	//--------------------------------------------------------------------------
	// Création d'objet PIX
	tcc_add_symbol(tcc, "pix_load_bmp", pix_load_bmp);
	tcc_add_symbol(tcc, "pix_load_jpg", pix_load_jpg);
	tcc_add_symbol(tcc, "pix_load_png", pix_load_png);
	tcc_add_symbol(tcc, "pix_load_raw", pix_load_raw);
	tcc_add_symbol(tcc, "pix_load_tga", pix_load_tga);

	tcc_add_symbol(tcc, "pix_save_bmp", pix_save_bmp);
	tcc_add_symbol(tcc, "pix_save_jpg", pix_save_jpg);
	tcc_add_symbol(tcc, "pix_save_png", pix_save_png);
	tcc_add_symbol(tcc, "pix_save_raw", pix_save_raw);
	tcc_add_symbol(tcc, "pix_save_tga", pix_save_tga);
	tcc_add_symbol(tcc, "pix_save_webp",pix_save_webp);

	tcc_add_symbol(tcc, "pix_create", pix_create);
	tcc_add_symbol(tcc, "pix_copy", pix_copy);
	tcc_add_symbol(tcc, "pix_destroy", pix_destroy);

	//--------------------------------------------------------------------------
	// Operations sur la disposition des pixels
	tcc_add_symbol(tcc, "pix_flip_vertical", pix_flip_vertical);
	tcc_add_symbol(tcc, "pix_flip_horizontal", pix_flip_horizontal);
	tcc_add_symbol(tcc, "pix_flip_horizontal_vertical", pix_flip_horizontal_vertical);

	tcc_add_symbol(tcc, "pix_redim", pix_redim);
	tcc_add_symbol(tcc, "pix_redimension", pix_redimension);
	tcc_add_symbol(tcc, "pix_extract", pix_extract);
	tcc_add_symbol(tcc, "pix_tile",	 pix_tile);

	//--------------------------------------------------------------------------
	//Opérations sur la couleur des pixels
	tcc_add_symbol(tcc, "pix_getPixel", pix_getPixel);
	tcc_add_symbol(tcc, "pix_setPixel", pix_setPixel);
	tcc_add_symbol(tcc, "getRed",   getRed);
	tcc_add_symbol(tcc, "getGreen", getGreen);
	tcc_add_symbol(tcc, "getBlue",  getBlue);

	tcc_add_symbol(tcc, "pix_fill", pix_fill);
	tcc_add_symbol(tcc, "pix_grayscale", pix_grayscale);
	tcc_add_symbol(tcc, "pix_invert_colors", pix_invert_colors);
	tcc_add_symbol(tcc, "pix_eclaircir", pix_eclaircir);
	tcc_add_symbol(tcc, "pix_color_filter", pix_color_filter);
	tcc_add_symbol(tcc, "pix_sepia", pix_sepia);
	tcc_add_symbol(tcc, "pix_blur", pix_blur);
	tcc_add_symbol(tcc, "pix_noise", pix_noise);
	//tcc_add_symbol(tcc, "pix_swap_RGB_vers_BGR",pix_swap_RGB_vers_BGR);
	tcc_add_symbol(tcc, "pix_difference", pix_difference);
	tcc_add_symbol(tcc, "pix_pixeliser", pix_pixeliser);

	//--------------------------------------------------------------------------
	// Dessiner des formes
	tcc_add_symbol(tcc, "pix_horizontal_line", pix_horizontal_line);
	tcc_add_symbol(tcc, "pix_vertical_line", pix_vertical_line);
	tcc_add_symbol(tcc, "pix_draw_line", pix_draw_line);
	tcc_add_symbol(tcc, "pix_circle", pix_circle);
	tcc_add_symbol(tcc, "pix_rectangle", pix_rectangle);
	tcc_add_symbol(tcc, "pix_square", pix_square);

	//--------------------------------------------------------------------------
	// Font et écriture de texte sur PIX
	tcc_add_symbol(tcc, "pix_load_font", pix_load_font);
	tcc_add_symbol(tcc, "pix_draw_text", pix_draw_text);
	tcc_add_symbol(tcc, "pix_draw_text_alpha", pix_draw_text_alpha);
	tcc_add_symbol(tcc, "pix_save_font", pix_save_font );
	tcc_add_symbol(tcc, "pix_destroy_font", pix_destroy_font);

	//--------------------------------------------------------------------------
	//  Blitting
	tcc_add_symbol(tcc, "pix_blit", pix_blit);
	tcc_add_symbol(tcc, "pix_blit_transluicid", pix_blit_transluicid);
	tcc_add_symbol(tcc, "pix_blit_transparent", pix_blit_transparent);
	tcc_add_symbol(tcc, "pix_AlphaChannelByColor",pix_AlphaChannelByColor);

	//--------------------------------------------------------------------------
	// Opérations pour la 3D et heightmaps
	tcc_add_symbol(tcc, "pix_blend", pix_blend);
	tcc_add_symbol(tcc, "pix_blending4", pix_blending4);
	tcc_add_symbol(tcc, "pix_bake", pix_bake);
	tcc_add_symbol(tcc, "pix_toNormalMap", pix_toNormalMap);
	tcc_add_symbol(tcc, "pix_circlesAlgorithm", pix_circlesAlgorithm);
	tcc_add_symbol(tcc, "pix_add", pix_add);
	tcc_add_symbol(tcc, "pix_lightmap", pix_lightmap);

	if (tcc_relocate(tcc, TCC_RELOCATE_AUTO) < 0) return 1;

	// invoque la fonction point d'entrée du script
	//int (*func)();
	void (*func)();
	func = tcc_get_symbol(tcc, "main");
	if (!func) { puts("main() introuvable"); return 1;}
	func();

	tcc_delete(tcc);
	return 0;
}

