/* ___ _      _ _ _     
  / _ (_)_  _| (_) |__  
 / /_)/ \ \/ / | | '_ \ 
/ ___/| |>  <| | | |_) |
\/    |_/_/\_\_|_|_.__/ */
#include <pixlib.h>
#include <stdlib.h>
#include <math.h>
//#ifdef WIN32
//#include <windows.h>
//#define PIXLIB_API _declspec (dllexport)
//€endif

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Placer un logo sur une image en x,y
    img    : pointeur PIX image. Modifié après appel de cette fonction.
    x,y    : position du logo à partir du coin supérieur gauche de l' image
    logo   : pointeur PIX logo
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_blit(PIX *img, int x, int y, PIX *logo)
{
	int top = img->height - logo->height - y;

	for(int lx=0; lx < logo->width; lx++)
		for(int ly=0; ly < logo->height; ly++)
		{
			long a = (lx+(ly*logo->width))*3; // position pixel logo
			long b = ((x+lx)+((top+ly)*img->width))*3; // position pixel image
			img->pixels[b+0] = logo->pixels[a+0]; // Modifie pixel image
			img->pixels[b+1] = logo->pixels[a+1];
			img->pixels[b+2] = logo->pixels[a+2];
		}
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Placer un logo transluicide sur une image en x,y.
    img   : pointeur sur pixbuf image 1. Modifié après appel.
    x,y   : position du logo à partir du coin supérieur gauche de l' image
    logo  : pointeur sur pixbuf image 2
    alpha : entre 0 et 1. 1 = le logo apparaitra opaque

  Le pixel du logo a un facteur de alpha.
  Celui de l'image a donc un facteur de (1 - transluicid).
  Ainsi pixel final = pixel_img*imgalpha + pixel_logo*alpha

         ex. logo.r = 100, img.r = 255 alpha = 0.1
            final.r = 100*0.1 + 255*0.9 = 10 + 229.5 = 239
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_blit_transluicid(PIX *img, int x, int y, PIX *logo, float alpha)
{
	float imgalpha = 1.0f - alpha;
	int top = img->height - logo->height - y;

	for(int lx=0; lx < logo->width; lx++)
		for(int ly=0; ly < logo->height; ly++)
		{
			// obtenir position pixel du logo
			long a = (lx+(ly*logo->width))*logo->nchannels;

			// position pixel de l'image
			long b = ((x+lx)+((top+ly)*img->width))*logo->nchannels;
			// Modifie pixel de l'image
			img->pixels[b+0] = (u8)(img->pixels[b+0]*imgalpha+logo->pixels[a+0]*alpha);
			img->pixels[b+1] = (u8)(img->pixels[b+1]*imgalpha+logo->pixels[a+1]*alpha);
			img->pixels[b+2] = (u8)(img->pixels[b+2]*imgalpha+logo->pixels[a+2]*alpha);
		}
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Placer un logo avec alpha ET transparence sur une image en x,y 
    img        : pointeur sur pixbuf image 1
    x,y        : position du logo à partir du coin supérieur gauche de l' image
    logo       : pointeur sur pixbuf image 2
    alpha      : entre 0 et 1.    1 -> le logo apparaitra opaque
    alphacolor : couleur du logo qui sera transparente au final

  Le pixel du logo a un facteur de alpha.
  Celui de l'image a donc un facteur de (1 - alpha).
  Ainsi pixel final = pixel_logo*alpha+pixel_img*alpha

         ex. logo.r = 100, img.r = 255 alpha = 0.1
            final.r = 100*0.1+255*0.9 = 10+229.5 = 239
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_blit_transparent(PIX *img, int x, int y, PIX *logo, float alpha, RGBcolor *transpaColor)
{
	float imgalpha = 1.0f - alpha;
	int top = img->height - logo->height - y;

	for(int lx=0; lx < logo->width; lx++)
		for(int ly=0; ly < logo->height; ly++)
		{
			long a = (lx+(ly*logo->width))*3; // position pixel logo

			// ignore le blanc! Le blanc sera transparent
			if (logo->pixels[a+0] == transpaColor->r &&
				logo->pixels[a+1] == transpaColor->g &&
				logo->pixels[a+2] == transpaColor->b) continue;

			long b = ((x+lx)+((top+ly)*img->width))*3; // position pixel image

			// Modifie pixel final
			img->pixels[b+0] = (u8)( img->pixels[b+0]*imgalpha + logo->pixels[a+0]*alpha );
			img->pixels[b+1] = (u8)( img->pixels[b+1]*imgalpha + logo->pixels[a+1]*alpha );
			img->pixels[b+2] = (u8)( img->pixels[b+2]*imgalpha + logo->pixels[a+2]*alpha );
		}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Blending.
  fusionner deux images selon un masque (image en niveaux de gris)

     img1: pointeur vers buffer PIX image 1. Modifié après appel.
     msk : pointeur vers buffer PIX du masque
     img2: pointeur vers buffer PIX image 2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_blend(PIX *img1, PIX *msk, PIX *img2)
{
	for(int x=0; x < img1->width; x++){
 		for(int y=0; y < img1->height; y++){
			long a = (x+(y*img1->width))*3;
			float f1 = (float)msk->pixels[a+0]/255.f;
			float f2 = 1 - f1;
			img1->pixels[a+0] = (u8)(img1->pixels[a+0]*f2 + img2->pixels[a+0]*f1);
			img1->pixels[a+1] = (u8)(img1->pixels[a+1]*f2 + img2->pixels[a+1]*f1);
			img1->pixels[a+2] = (u8)(img1->pixels[a+2]*f2 + img2->pixels[a+2]*f1);
		}
	}
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Blending.

  fusionner 4 images selon une image en niveaux de gris.

  Idéal pour le terrain: la mer au plus bas, puis sable/herbe/neige.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*------------------------------------------------------------------------------
   get for percentage of color
------------------------------------------------------------------------------*/
float texture_factor(int h1, float h2, int step)
{
	float t = ((float)step - abs(h1 - (int)h2)) / (float)step;
	if     (t < 0.0f) t = 0.0f;
	else if(t > 1.0f) t = 1.0f;
	return t;
}
/*----------------------------------------------------------------------------*/
PIX* pix_blending4(PIX* msk, const char* ftx1, const char* ftx2, const char* ftx3, const char* ftx4)
{
	//printf("ok %d %d\n",msk->width,msk->height);
	//printf("tx:\n\t%s\n\t%s\n\t%s\n\t%s\n",ftx1,ftx2,ftx3,ftx4);

	PIX* tx[4];
		tx[0] = pix_load_bmp(ftx1);
		tx[1] = pix_load_bmp(ftx2);
		tx[2] = pix_load_bmp(ftx3);
		tx[3] = pix_load_bmp(ftx4);

	PIX* final = pix_create(msk->width,msk->height, 3);

	u8 nbTextures = 4;
	int step = 256 / nbTextures;

	//float texFactor;	// texture factor of each texture to blend
	int alt[4] = {0, 85, 170, 256}; // took me time to get this one
	float red, green, blue; 	// final rgb at x,y

	// For each pixel
	unsigned short x, y;

	//printf("loop\n"); getchar();

  for(y=0; y<msk->height; y++)
  {
    for(x=0; x<msk->width; x++)
    {
      //Get altitude from heightmap at x/z
      RGBpixel alti;
      pix_getPixel(msk, x,y, &alti);

      red = green = blue = 0;
      u8 i;
      for (i=0; i<nbTextures; i++) // For each texture to blend,
      {
        //if one of the images is smaller, repeat it
        int kx = x, ky = y; //where to read

        if (x >= tx[i]->width)
        {
          unsigned short nx = (int)(x / tx[i]->width);
          kx = x - (nx * tx[i]->width);
          //tx = x - ((int)(x / B[i].getWidth()) * B[i].getWidth());
        }

        if (y >= tx[i]->height)
        {
          unsigned short ny = (int)(y / tx[i]->height);
          ky = y - (ny * tx[i]->height);
        }

        // Read its rgb value ...
        RGBpixel color;
        pix_getPixel(tx[i], kx,ky, &color);

        // ... to determine texture factor of this texture depending on altitude
        float texFactor = texture_factor(alt[i], alti.r, step);

        //add to new RGB value for the pixel to be written
        red   += color.r * texFactor;
        green += color.g * texFactor;
        blue  += color.b * texFactor;
      }

      // apply new pixel to final destination
      //final.setcolor(x, y, (unsigned char)red, (unsigned char)green, (unsigned char)blue);
      pix_setPixel(final, x,y, (u8)red, (u8)green, (u8)blue);
    }
 }
	//printf("end loop\n"); getchar();
	for (u8 i=0; i<nbTextures; i++) pix_destroy(tx[i]);
	//printf("final:%d,%d\n", final->width, final->height); getchar();
	return final;
}
/*==============================================================================
  superpose deux heightmaps. b est modifié après appel
==============================================================================*/
void pix_add(PIX* a, PIX* b)
{
	for(int y=0; y<a->height; y++)
	{
		for (int x=0; x<a->width; x++)
		{
			u8 *p1 = a->pixels + (x + y * a->width) *a->nchannels;
			u8 *p2 = b->pixels + (x + y * b->width) *b->nchannels;
			if (p1[0]!=0)
			{
				if (p2[0]!=0)
					p2[0] = p2[1] = p2[2] = (u8)((p1[0]+p2[0])/2);
				else
					p2[0] = p2[1] = p2[2] = p1[0];
			}
		}
	}
}
/*==============================================================================
  Crée et retourne un PIX 4 channels à partir d'un PIX 3 channels dont une
  couleur sera transparente au final.
  Le PIX retourné doit être libéré après utilisation pour sauver de la mémoire
  et img peut devenir inutile aussi, en tout cas il est double en mémoire après
  l'appel
==============================================================================*
PIX* AlphaByColor(PIX* img, RGBcolor* color)
{
  PIX *pix = pix_create(img->width, img->height, 4);

  int x, y;
  for(y=0; y<img->height; y++)
  {
    for(x=0; x<img->width; x++)
    {
      u8* i = img->pixels + (x+ y * img->width) * img->nchannels;
      u8* p = pix->pixels + (x+ y * pix->width) * pix->nchannels;

      p[0] = i[0]; //R
      p[1] = i[1]; //G
      p[2] = i[2]; //B

      if (i[0] == color->r &&
          i[1] == color->g &&
          i[2] == color->b )
        p[3] = 255; //A
    }
  }
  return pix;
}
/ *==============================================================================
  Crée et retourne le canal alpha (PIX 1 channel) à partir d'un PIX 3 channels
  dont une couleur sera transparente au final.
  Le PIX retourné doit être libéré après utilisation pour sauver de la mémoire
==============================================================================*/
PIX* pix_AlphaChannelByColor(PIX* img, RGBcolor* color)
{
	PIX *msk = pix_create(img->width, img->height, 1);//3
	RGBcolor c = {255,255,255}; // le remplit de blanc
	pix_fill(msk, &c);

	for(int y=0; y<img->height; y++)
	{
		for(int x=0; x<img->width; x++)
		{
			u8* i = img->pixels + (x+ y * img->width) * img->nchannels;
			u8* p = msk->pixels + (x+ y * msk->width);// * img->nchannels;

			if (i[0] == color->r &&
				i[1] == color->g &&
				i[2] == color->b )
				//p[0] = p[1] = p[2] = 255;
				p[0] = 0;//255;
		}
	}
	return msk;
}

