/* ___ _      _ _ _     
  / _ (_)_  _| (_) |__  
 / /_)/ \ \/ / | | '_ \ 
/ ___/| |>  <| | | |_) |
\/    |_/_/\_\_|_|_.__/ */
// Cet objet contient toutes les fonctions qui modifient la couleur des pixels.
#include <pixlib.h>
#include <stdlib.h>
#include <math.h>

//#define PIXLIB_API _declspec (dllexport)

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Transforme une image en niveau de gris.
     img: pointeur vers un buffer PIX qui sera modifié après appel
  Toute couleur grise a une particularité: ses trois valeur RGB sont égales.
  Il suffit donc de faire la moyenne du RGB de chaque pixel.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_grayscale(PIX *img)
{
	for(int y=0; y < img->height; y++)
	{
		for(int x=0; x < img->width; x++)
		{
			u8 *p = img->pixels + (x+ y * img->width) * img->nchannels;
			p[0] = p[1] = p[2] = (u8)((p[0] + p[1] + p[2])/3);
		}
	}
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Inverser les couleurs
     img: pointeur vers buffer PIX qui sera modifié après appel
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_invert_colors(PIX *img)
{
	for(int y=0; y < img->height; y++)
	{
		for(int x=0; x < img->width; x++)
		{
			u8 *p = img->pixels + (x+ y * img->width) * img->nchannels;
			p[0] = 255 - p[0];
			p[1] = 255 - p[1];
			p[2] = 255 - p[2];

			/*a = (x+(y*img->width))*3;
			img->pixels[a+0] = 255 - img->pixels[a+0];
			img->pixels[a+1] = 255 - img->pixels[a+1];
			img->pixels[a+2] = 255 - img->pixels[a+2];*/
		}
	}
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Remplit un pixbuf contenant les différences entre deux images

     img1 : pointeur vers buffer BMP image1. Modifié après appel.
     img2 : pointeur vers buffer BMP image2
     final: pointeur vers buffer final

  Retourne en plus le nombre d'octets qui sont différents.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
long pix_difference(PIX *img1, PIX *img2, PIX *final)
{
	long nb_differences=0;

	for(int y=0; y < img1->height; y++)
	{
		for(int x=0; x < img1->width; x++)
		{
			long a = (x+(y*img1->width))*3;

			// si le pixel est différent, le met sur le pixel final
			if (img1->pixels[a+0] != img2->pixels[a+0] ||
				img1->pixels[a+1] != img2->pixels[a+1] ||
				img1->pixels[a+2] != img2->pixels[a+2])
			{
				nb_differences++;
				final->pixels[a+0] = img2->pixels[a+0];
				final->pixels[a+1] = img2->pixels[a+1];
				final->pixels[a+2] = img2->pixels[a+2];
			}
		}
	}
	return nb_differences;
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Eclaircir ou assombrir une image.

     img : pointeur vers buffer BMP image1. Modifié après appel.
     i   : si i > 0 éclaircir sinon assombrir
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_eclaircir (PIX *img, int i)
{
	if (!img || i==0) return;
	if (img->nchannels != 3) return;

	if (i > 0) // i est potisif: éclaircir
	{
		for(int y=0; y < img->height; y++)
		{
			for(int x=0; x < img->width; x++)
			{
				u8 *p = img->pixels + (x+ y * img->width) * img->nchannels;
				p[0]=((p[0]+i) > 255)? 255 : p[0]+i;
				p[1]=((p[1]+i) > 255)? 255 : p[1]+i;
				p[2]=((p[2]+i) > 255)? 255 : p[2]+i;
			}
		}
	}
	else // i est négatif: assombrir
	{
		for(int y=0; y < img->height; y++)
		{
			for(int x=0; x < img->width; x++)
			{
				u8 *p = img->pixels + (x+ y * img->width) * img->nchannels;
				p[0]=((p[0]+i) < 0)? 0 : p[0]+i;
				p[1]=((p[1]+i) < 0)? 0 : p[1]+i;
				p[2]=((p[2]+i) < 0)? 0 : p[2]+i;
			}
		}
	}
	
	// else if 4 channels
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Applique un filtre de couleur.
  Passer RGBcolor c = {0,0,255} rendra l'image bleue
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_color_filter(PIX *img, RGBcolor *c)
{
	float fr = (float)c->r / 255.f;
	float fg = (float)c->g / 255.f;
	float fb = (float)c->b / 255.f;

	for(int y=0; y < img->height; y++)
	{
		for(int x=0; x < img->width; x++)
		{
			u8 *p = img->pixels + (x+ y * img->width) * img->nchannels;
			u8 moyenne = (u8)((p[0] + p[1] + p[2])/3);
			p[0]=(u8)(moyenne * fr);
			p[1]=(u8)(moyenne * fg);
			p[2]=(u8)(moyenne * fb);
		}
	}
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Effet Sepia (genre de noir et blanc mais marron)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_sepia (PIX *img)
{
	RGBcolor color = {203,177,78};
	pix_color_filter(img, &color);
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Flou
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_blur (PIX *img)
{
	// Flou horizontal
	for(int y=0; y < img->height; y++)
		for(int x=1; x < img->width; x++)
		{
			u8 *p1 = img->pixels + (x-1+ y * img->width) * img->nchannels;
			u8 *p2 = img->pixels + (x+ y * img->width) * img->nchannels;
			p1[0] = p2[0] = (u8)((p1[0]+p2[0])/2);
			p1[1] = p2[1] = (u8)((p1[1]+p2[1])/2);
			p1[2] = p2[2] = (u8)((p1[2]+p2[2])/2);
		}
	// Flou vertical
	for(int y=1; y < img->height; y++)
		for(int x=0; x < img->width; x++)
		{
			u8 *p1 = img->pixels + (x+ (y-1) * img->width) * img->nchannels;
			u8 *p2 = img->pixels + (x+ y * img->width) * img->nchannels;
			p1[0] = p2[0] = (u8)((p1[0]+p2[0])/2);
			p1[1] = p2[1] = (u8)((p1[1]+p2[1])/2);
			p1[2] = p2[2] = (u8)((p1[2]+p2[2])/2);
		}
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Remplir un PIX d'une couleur unie
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_fill(PIX *img, RGBcolor *c)
{
	for(int y=0; y < img->height; y++)
	{
		for(int x=0; x < img->width; x++)
		{
			u8 *p = img->pixels + (x+ y * img->width) * img->nchannels;
			p[0] = c->r;
			p[1] = c->g;
			p[2] = c->b;
		}
	}
}
/*==============================================================================
  Implemented from http://www.lighthouse3d.com/opengl/terrain/index.php3?circles

  for each terrain point (tx,tz) do

	pd = distance from circle center * 2 / terrainCircleSize

	if fabs(pd) <= 1.0

		height(tx,tz) +=  disp/2 + cos(pd*3.14)*disp/2;

  where terrainCircleSize defines the circle size and disp defines the maximum
  height variation. As in the fault algorithm, the value of disp can be decreased
  as the number of iterations increases.
==============================================================================*/
void pix_circlesAlgorithm(PIX* img, int iterations, int radius, int disp, int ax, int ay)
{
	radius = radius/2 + rand()%radius;
	disp   = disp/2   + rand()%disp;

	for (int n=0; n<iterations; n++)
	{
		//system("CLS");
		//printf("%d/%d\n", n, iterations);

		// pseudo random point near ax,ay
		int cx = ax + rand()%radius;
		int cy = ay + rand()%radius;

		for(int y=0; y < img->height; y++)
		{
			for(int x=0; x < img->width; x++)
			{
				double pd = sqrt(pow(y - cy, 2) + pow(x - cx, 2)) * 2 / radius;
				if (fabs(pd) <= 1.0)
				{
					int val = (int)(disp / 2 + cos(pd*3.14) * disp / 2);
					u8* p = img->pixels + (x+ y * img->width) * img->nchannels; //1 chan
					p[0] = p[1] = p[2] = (p[0] + val < 256) ? p[0] + val : 255;
				}
			}
		}
	}
}
/*==============================================================================
  replace a pix by its normal map. 2.f is good for factor but i often use 128
==============================================================================*/
void pix_toNormalMap(PIX* img, int factor)
{
	float V1[3]; // vecteur sur l'axe Y
	float V2[3]; // vecteur sur l'axe X
	float normal[3];
	//int channels = img->nchannels;//getBpp()/8;

	// pointeur vers les pixels courants, 0 = pixel "central" 1 = pixel du haut, 2 = pixel de droite
	u8 *pixel[3];

	u8 *newBuffer = malloc(img->height*img->width*img->nchannels);

	for (int i = 0; i < img->width; i++)
	{
		for (int j = 0; j < img->height; j++)
		{
			int pos1 = (i*img->height + j)*img->nchannels;
			int newPos1 = (i*img->height + j)*img->nchannels;
			int pos2 = (((i+1)%img->width)*img->height + j)*img->nchannels;
			int pos3 = (i*img->height + ((j+1)%img->height))*img->nchannels;
			// determination des pixels courants
			pixel[0] = &img->pixels[pos1];
			pixel[1] = &img->pixels[pos3];
			pixel[2] = &img->pixels[pos2];
			// pour chaque pixel  crée les vecteurs "unitaire" sur les axes X et Y
			V1[0] = 0;
			V1[1] = 1;
			// raporte la hauteur a une unité, la plage possible etant [0..255].
			//   prend la coordonnée Z de la couleur
			V1[2] = (((int)*pixel[1] - (int)*pixel[0])/255.f) * factor;
			V2[0] = 1;
			V2[1] = 0;
			// raporte la hauteur a une unité, la plage possible etant  [0..255]
			V2[2] = (((int)*pixel[2] - (int)*pixel[0])/255.f) * factor;

			// calcule du vecteur normal
			normal[0] = V2[1] * V1[2] - V2[2] * V1[1];
			normal[1] = V2[2] * V1[0] - V2[0] * V1[2];
			normal[2] = V2[0] * V1[1] - V2[1] * V1[0];

			// normalisation du vecteur normal
			float len = sqrt(normal[0]*normal[0] + normal[1]*normal[1] + normal[2]*normal[2]);
			normal[0]/=len;
			normal[1]/=len;
			normal[2]/=len;

			/* pour doom3 remplacer
			newBuffer[newPos1+1] = normal[0]*127 + 128;
			par
			newBuffer[newPos1+1] =255-( normal[0]*127 + 128);
			*/
			newBuffer[newPos1]   = (u8)(normal[1]*127 + 128);
			newBuffer[newPos1+1] = (u8)(normal[0]*127 + 128);
			//newBuffer[newPos1+1] =255-( normal[0]*127 + 128);
			newBuffer[newPos1+2] = (u8)(normal[2]*127 + 128);
		}
	}
	// swap buffers
	free(img->pixels);
	img->pixels = newBuffer;
}
/*==============================================================================
  add noise to a map. Range 0-255 for strength.
==============================================================================*/
void pix_noise(PIX* img, int strength)
{
	unsigned short x, y;

	if (img->nchannels == 3)
	{
		for(y=0; y<img->height; y++)
		{
			for(x=0; x<img->width; x++)
			{
				u8 *p = img->pixels + (x+ y * img->width) * img->nchannels; 
				u8 r = rand()%strength;
				if (p[0] + r < 256)
				{
					u8 v = p[0] + r;
					p[0] = p[1] = p[2] = v;
				}
			}
		}
	}
	else if (img->nchannels == 1)
	{
		for(y=0; y<img->height; y++)
		{
			for(x=0; x<img->width; x++)
			{
				u8 *p = img->pixels + (x+ y * img->width); 
				u8 r = rand()%strength;
				if (p[0] + r < 256) p[0] += r;
			}
		}
	}
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Add Lightmap
  ambient : couleur sera composante gris RGB de l'endroit pleinement ombré
  angle_rayons : angle vertical ou horizontal???
  hauteur_max_rayons : c'est quoi sa aussi?
  hauteur_max_heightmap : altitude réelle en mètres du point le plus élevé

  //Commentaires de funto
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
PIX* pix_lightmap(PIX* img, int ambient, float angle_rayons,
			   float hauteur_max_rayons, float hauteur_max_heightmap)
{
  int x, y;
  float step_z=0.0f;
  float z_origine=0.0f;
  int x_rayon=0;
  float z_rayon=0.0f;


  PIX* lmap = pix_create(img->width, img->height, img->nchannels);

  // Au départ, tout est dans l'ombre...
  for(y=0 ; y<lmap->height ; y++)
    for(x=0 ; x<lmap->width ; x++)
    {
      u8 *p = lmap->pixels  + (x + y * lmap->width) *lmap->nchannels;
      p[0] = p[1] = p[2] = ambient;
    }

  // Puis vint la lumière !!
  // calcule de combien on doit incrémenter en z à chaque nouveau rayon
  step_z = fabs(1/tan(angle_rayons));


  // lance les rayons dans le sens horizontal (en voyant de haut), de la
  //   gauche vers la droite. fait donc ça pour toutes les lignes (d'où la
  //   boucle en y). 
  for(y=0 ; y<lmap->height ; y++)
  {
    // parcourt en montant (i.e. suivant l'axe des Z) pour lancer des rayons
    // à chacune de ces hauteurs
    for(z_origine=0.0f ; z_origine<hauteur_max_rayons ; z_origine+=step_z)
    {
      // Lancer du rayon !
      for(z_rayon=z_origine, x_rayon=0;
			x_rayon < lmap->width;
			z_rayon-=step_z, x_rayon++)
      {
        // Si la hauteur du pixel en cours est supérieure ou égale à z_rayon,
        //   c'est qu'on a touché qqch, alors on le colorie en blanc et on passe
        //   au rayon d'après

        // pointe sur le point (x_rayon, y)
        u8 *p1 = img->pixels  + (x_rayon + y * img->width) *img->nchannels;
        u8 *p2 = lmap->pixels + (x_rayon + y * lmap->width) *lmap->nchannels;

        if((float)(p1[0]/255.0f)*hauteur_max_heightmap >= z_rayon)
        {
           p2[0] = p2[1] = p2[2] = 255;
           break;
        }
      }
    }
  }

  return lmap;
}
/*==============================================================================
  b will be changed : a (AO or lightmap) will be put on it, multiplied
  a can have only one channel, but b must have 3.
==============================================================================*/
void pix_bake(PIX *a, PIX *b)
{
	for(int y=0; y<a->height; y++)
		for (int x=0; x<a->width; x++)
		{
			u8 *p1 = a->pixels + (x + y * a->width) *a->nchannels;
			u8 *p2 = b->pixels + (x + y * b->width) *b->nchannels;
			float f = p1[0] / 255.f;
			p2[0] = (u8)(p2[0] * f);
			p2[1] = (u8)(p2[1] * f);
			p2[2] = (u8)(p2[2] * f);
		}
}
/*==============================================================================
  img modifiée après appel. factor détermine la taille des "pixels" de 0 à 255
==============================================================================*/
void pix_pixeliser(PIX *img, u8 factor)
{
	short unsigned int divisions = img->width / factor;
	short int step = img->width / divisions;

	for (int y=0; y<=divisions; y++)
	{
		short int y1 = y*step;
		short int y2 = (y+1)*step;

		for (int x=0; x<=divisions; x++)
		{
			short int x1 = x*step;
			short int x2 = (x+1)*step;

			int w = x2-x1;
			int h = y2-y1;
			long moyenne[3] = {0,0,0};
			int nb=0; // calcule la moyenne de ce tile
			for (int tx=0; tx<w; tx++){
				for(int ty=0; ty<h; ty++){
					u8 *p = img->pixels  + ((tx+x1) + (ty+y1) * img->width) *img->nchannels;
					moyenne[0] += p[0];
					moyenne[1] += p[1];
					moyenne[2] += p[2];
					nb++;
				}
			}

			u8 moyr = (u8)(moyenne[0] / nb); // applique la moyenne partour dans ce tile
			u8 moyg = (u8)(moyenne[1] / nb);
			u8 moyb = (u8)(moyenne[2] / nb);
			for (int tx=0; tx<w; tx++){
				for(int ty=0; ty<h; ty++){
					u8 *p = img->pixels  + ((tx+x1) + (ty+y1) * img->width) *img->nchannels;
					p[0] = moyr;
					p[1] = moyg;
					p[2] = moyb;
				}
			}
		}
	}
}

