/* ___ _      _ _ _     
  / _ (_)_  _| (_) |__  
 / /_)/ \ \/ / | | '_ \ 
/ ___/| |>  <| | | |_) |
\/    |_/_/\_\_|_|_.__/ */
// Cet objet contient toutes les fonctions qui modifient la disposition des pixels.
#include <pixlib.h>
//#define PIX_API _declspec (dllexport) 

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	Redim en précisant la largeur/hauteur finale
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_redimension (PIX *img, unsigned short int width, unsigned short int height)
{
	PIX *tmp; // PIX temporaire à la taille voulue
	float tauxX, tauxY;

	if (!img || (img->width == width && img->height==height)) return;

	tmp = pix_create(width, height, img->nchannels);

	tauxX  = (float)( (img->width  * 100.f) / (float)width);
	tauxY  = (float)( (img->height * 100.f) / (float)height);

	//printf("w/h\t\t:%d,%d\n", img->width, img->height);
	//printf("final w/h\t:%d,%d\n", width, height);
	//printf("taux x/y\t:%.2f,%.2f\n", tauxX, tauxY);

	// Parcourt le nouveau et va chercher le pixel correspondant sur l'original
	for (int y=0; y<tmp->height; y++)
		for (int x=0; x<tmp->width; x++)
		{
			long a = (x + y * tmp->width) * tmp->nchannels;
			long b = ( (int)((x*tauxX)/100.f) + (int)((y*tauxY)/100.f) * img->width) * img->nchannels;
			tmp->pixels[a+0] = img->pixels[b+0];
			tmp->pixels[a+1] = img->pixels[b+1];
			tmp->pixels[a+2] = img->pixels[b+2];
			//printf("%d    x,y:%d/%d   xtx,y:%d,%d\n", cpt++, x,y, x*tauxX/100.f,y*tauxY/100.f);
		}

	// détruit l' original et le remplace par le nouveau
	pix_destroy(img);
	img = pix_copy(tmp);
	pix_destroy(tmp);
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Redimensionner un PIX
   img      : pointeur sur pix à redimensionner
   pourcentX: pourcentage à appliquer à la largeur
   pourcentY: pourcentage à appliquer à la heuteur

   Un pourcentage entre 0 et 99 réduit la taille
   Un pourcentage à 100 laisse identique
   Un pourcentage > 100 agrandit
   
   ex. PIX 300X200 pixels, redim 70%,70%
     largeur image finale -> 300*70/100 = 210 pixels
     hauteur image finale -> 200*70/100 = 140 pixels
     70% -> on garde 1 pixel sur 100/70 = 1,4286
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_redim (PIX *img, unsigned int pourcentX, unsigned int pourcentY)
{
	if (!img) return;
	if (pourcentX==100 && pourcentY==100) return;

	PIX *tmp = pix_create((int)(img->width * (pourcentX / 100.f)),
                   (int)(img->height * (pourcentY / 100.f)), img->nchannels);

	float tauxX  = (float)(100.f / (float)pourcentX);
	float tauxY  = (float)(100.f / (float)pourcentY);

	// Parcourt le nouveau et va chercher le pixel correspondant sur l'original
	for (int y=0; y<tmp->height; y++)
		for (int x=0; x<tmp->width; x++)
		{
 			long a = (x + y * tmp->width) * tmp->nchannels;
			long b = ( (int)(x*tauxX) + (int)(y*tauxY) * img->width) * img->nchannels;
			tmp->pixels[a+0] = img->pixels[b+0];
			tmp->pixels[a+1] = img->pixels[b+1];
			tmp->pixels[a+2] = img->pixels[b+2];
		}

	// détruit l' original et le remplace par le nouveau
	pix_destroy(img);
	img = pix_copy (tmp);
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Retourne horizontalement et verticalement une image.
     img  : pointeur vers buffer BMP image1
     dest : buffer qui contiendra les différences après appel. Doit être de
            mêmes dimensions que l' image.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_flip_horizontal_vertical(PIX *img)
{
	if (!img) return;
	PIX *tmp = pix_copy(img);

	for(int y=0; y < img->height; y++)
	{
		for(int x=0; x < img->width; x++)
		{
			long a = (x + (y*img->width))*3;
			long b = ((img->width-x) + ((img->height-y-1)*img->width))*3;
			img->pixels[a+0]=tmp->pixels[b+0];
			img->pixels[a+1]=tmp->pixels[b+1];
			img->pixels[a+2]=tmp->pixels[b+2];
		}
	}
	pix_destroy(tmp);
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Retourne horizontalement une image.
     img : pointeur vers buffer BMP image1
     dest : buffer qui contiendra les différences après appel. Doit être de
            mêmes dimensions que l' image.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_flip_horizontal(PIX *img)
{
	if (!img) return;
	PIX *tmp = pix_copy(img);

	for(int y=0; y < img->height; y++)
	{
		for(int x=0; x < img->width; x++)
		{
			long a = (x + (y*img->width))*3;
			long b = ((img->width-x) + (y*img->width))*3;
			img->pixels[a+0]=tmp->pixels[b+0];
			img->pixels[a+1]=tmp->pixels[b+1];
			img->pixels[a+2]=tmp->pixels[b+2];
		}
	}
	pix_destroy(tmp);
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Retourne et verticalement une image
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_flip_vertical(PIX *img)
{
	if (!img) return;
	PIX *tmp = pix_copy(img);

	for(int y=0; y < img->height; y++)
	{
		for(int x=0; x < img->width; x++)
		{
			long a = (x + (y*img->width))*3;
			long b = (x + ((img->height-y-1)*img->width))*3;
			img->pixels[a+0]=tmp->pixels[b+0];
			img->pixels[a+1]=tmp->pixels[b+1];
			img->pixels[a+2]=tmp->pixels[b+2];
		}
	}
	pix_destroy(tmp);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Crée un PIX en extrayant un rectangle d'une image
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
PIX *pix_extract(PIX *img, int x1, int y1, int x2, int y2)
{
	if (x2>img->width || y2>img->height || x1>x2 || y1>y2) return NULL;

	PIX *tmp = pix_create(x2-x1,y2-y1, 3);

	for (int x=0; x<tmp->width; x++)
		for(int y=0; y<tmp->height; y++)
		{
			u8 *p2 = tmp->pixels + (  x    +   y    * tmp->width)*tmp->nchannels;
			u8 *p1 = img->pixels + ((x+x1) + (y+y1) * img->width)*img->nchannels;
			p2[0]=p1[0];
			p2[1]=p1[1];
			p2[2]=p1[2];
    }
	return tmp;
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  decouper et exporter n = divisions X divisions images
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_tile(PIX* img, short unsigned int divisions, const char* folder)
{
	short int step = img->width / divisions;

	for (int y=0; y<divisions; y++)
	{
		short int y1 = y*step;
		short int y2 = (y+1)*step;

		for (int x=0; x<divisions; x++)
		{
			// create and fill the tile, then destroy it
			short int x1 = x*step;
			short int x2 = (x+1)*step;

			PIX *tile = pix_create(x2-x1,y2-y1, 3); // why not only one channel

			for (int tx=0; tx<tile->width; tx++)
				for(int ty=0; ty<tile->height; ty++)
				{
					u8 *p2 = tile->pixels + (  tx    +   ty    * tile->width)*tile->nchannels;
					u8 *p1 = img->pixels  + ((tx+x1) + (ty+y1) * img->width) *img->nchannels;
					p2[0]=p1[0];
					p2[1]=p1[1];
					p2[2]=p1[2];
				}

			char file[64];
			sprintf(file,"%s/tile%02d.%02d.png", folder, x, y);
			pix_save_png(tile, file);
			pix_destroy(tile);
		}
	}
}

