/* ___ _      _ _ _     
  / _ (_)_  _| (_) |__  
 / /_)/ \ \/ / | | '_ \ 
/ ___/| |>  <| | | |_) |
\/    |_/_/\_\_|_|_.__/ */
#include <pixlib.h>
#include <stdlib.h>
#include <math.h>
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  draw a vertical line between two points
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_vertical_line(PIX* img, int x1, int y1, int y2, RGBcolor *c)
{
	int y, ys, ye;// y start/end
	if (y2>y1)
	{
		ys = y1;
		ye = y2;
	}
	else
	{
		ys = y2;
		ye = y1;
	}

	for (y=ys; y<ye; y++)
	{
		u8 *p = img->pixels + (x1 + (img->height - y) * img->width) *img->nchannels;
		p[0] = c->r;
		p[1] = c->g;
		p[2] = c->b;
	}
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  draw a line between two points
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_horizontal_line(PIX* img, int x1, int y1, int x2, RGBcolor *c)
{
	//equation de la droite passant par les deux points
	//float a = 0.f / (float)(x2-x1);
	//float b = y1 - a*x1;

	int x, xs, xe;// x start/end
	if (x2>x1)
	{
		xs = x1;
		xe = x2;
	}
	else
	{
		xs = x2;
		xe = x1;
	}

	for (x=xs; x<xe; x++)
	{
		u8 *p = img->pixels + (x + (img->height - y1) * img->width) *img->nchannels;
		p[0] = c->r;
		p[1] = c->g;
		p[2] = c->b;
	}
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Bresenham line from (x1,y1) to (x2,y2) with rgb color
  http://www.ele.uva.es/~jesman/BigSeti/ftp/Instalacion/Software/Milling/QuickCG.cpp
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_draw_line(PIX* img, int x1, int y1, int x2, int y2, RGBcolor *c)
{
	int w = img->width;
	int h = img->height;
	if (x1 < 0 || x1 > w - 1 || x2 < 0 || x2 > w - 1 ||
		y1 < 0 || y1 > h - 1 || y2 < 0 || y2 > h - 1) return;
    
	int deltax = abs(x2 - x1);
	int deltay = abs(y2 - y1);
	int x = x1;
	int y = y1;
	int xinc1, xinc2, yinc1, yinc2, den, num, numadd, numpixels, curpixel;

	if (x2 >= x1)
	{
		xinc1 = 1;
		xinc2 = 1;
	}
	else
	{
		xinc1 = -1;
		xinc2 = -1;
	}
	if (y2 >= y1)
	{
		yinc1 = 1;
		yinc2 = 1;
	}
	else
	{
		yinc1 = -1;
		yinc2 = -1;
	}
	if (deltax >= deltay)
	{
		xinc1 = 0;
 		yinc2 = 0;
		den = deltax;
		num = deltax / 2;
		numadd = deltay;
		numpixels = deltax;
	}
	else
	{
		xinc2 = 0;
		yinc1 = 0;
		den = deltay;
		num = deltay / 2;
		numadd = deltax;
		numpixels = deltay;
	}

	for (curpixel = 0; curpixel <= numpixels; curpixel++)
	{
		pix_setPixel(img, x % w, h - (y % h), c->r,c->g,c->b);

		num += numadd;
		if (num >= den)
		{
			num -= den;
			x += xinc1;
			y += yinc1;
		}
		x += xinc2;
		y += yinc2;
	}
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  déterminer equation de droite. boucler du x point 1 au 2 trouver y avec equation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*
void pix_line(PIX* img, int x1, int y1, int x2, int y2,
                         u8 largeur, RGBcolor *color)
{
  int n;
  for (n=0; n<largeur; n++)
  {
    x1++;
    x2++;

    // Equation of a line is y = ax + b. a est la pente:
    float a = (y2-y1) / (x2-x1);
    // so b = y - a*x . Get b with coords of a point:
    float b = y1 - a*x1;
    //now we can use y = a*x + b between the two points

    int y;
    for (y=y1; y<y2; y++)
    {
      //int y = a*x + b;
      int x = (y - b)/a;
      u8 *p = img->pixels + (x + y * img->width) *img->nchannels;
      p[0] = color->r;
      p[1] = color->g;
      p[2] = color->b;
    }
  }
}
/ *%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 For each point if it's distance to the center (Pythagoras's) is less than/equals
 radius it's in the circle, otherwise it is outside the circle
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_circle(PIX* img, int cx, int cy, int radius, RGBcolor* color)
{
	int fx = cx + radius, fy = cy + radius;
	// start from a point close to the circle, dont traverse whole bitmap!
	for(int y=cy-radius; y<fy; y++)
		for (int x=cx-radius; x<fx; x++)
		{
			// distance from that point to center
			float dist = sqrt(pow(x-cx, 2) + pow(y-cy, 2));
			if (dist <= radius)
			{
				u8 *p = img->pixels + (x + y * img->width) *img->nchannels;
				p[0] = color->r;
				p[1] = color->g;
				p[2] = color->b;
			}
		}
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  draw a red rectangle
  on peut faire plus rapide que faire plein de setPixel
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_rectangle(PIX* img, int x1, int x2, int y1, int y2, RGBcolor *c)
{
	int i, y;
	for(i=x1;i<x2;i++) pix_setPixel(img, i, y1, c->r,c->g,c->b);
	for(y=y1;y<y2;y++) pix_setPixel(img, x1, y, c->r,c->g,c->b);
	for(i=x1;i<x2;i++) pix_setPixel(img, i, y2, c->r,c->g,c->b);
	for(y=y1;y<y2;y++) pix_setPixel(img, x2, y, c->r,c->g,c->b);					
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  draw a square
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_square(PIX* img, int x1, int y1, int length, RGBcolor *c)
{
	int i, y;
	for(i=x1;i<x1+length;i++) pix_setPixel(img, i, y1, c->r,c->g,c->b);
	for(y=y1;y<y1+length;y++) pix_setPixel(img, x1, y, c->r,c->g,c->b);
	for(i=x1;i<x1+length;i++) pix_setPixel(img, i, y1+length, c->r,c->g,c->b);
	for(y=y1;y<y1+length;y++) pix_setPixel(img, x1+length, y, c->r,c->g,c->b);
}

