/*******************************************************************************
   ___ _      _ _ _     
  / _ (_)_  _| (_) |__  
 / /_)/ \ \/ / | | '_ \ 
/ ___/| |>  <| | | |_) |
\/    |_/_/\_\_|_|_.__/ 

 ** Copyright (C) 2008-2021 Robin Masta

 ** This software is provided 'as-is', without any express or implied
 ** warranty.  In no event will the authors be held liable for any damages
 ** arising from the use of this software.

 ** Permission is not granted to anyone to use this software for any purpose,
 ** including private and commercial applications, and to alter it and
 ** redistribute it.
 ******************************************************************************/
#include <pixlib.h>
#include <stdlib.h>
#include <string.h>
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Crée un PIXFONT manuellement à partir d'un bmp
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*
  int x;
  FILE *pF = fopen("ascii.txt", "w");
  for(x=0; x<256; x++)
    fprintf(pF, "%d %c\n", x, x);
  fclose(pF);
  return 0;
*/
PIXFONT *testfont()
{
	PIXFONT *font;
	PIX *tmp;

	font = (PIXFONT*) malloc(sizeof(PIXFONT));
	font->taille            = 7;
	font->largeur_caractere = 6;
	font->hauteur_caractere = 10;

	tmp = pix_load_bmp("bande6.bmp");
	font->width             = tmp->width;
	font->pixels            = (unsigned char*) malloc(tmp->width * tmp->height);

	// passe pixels du PIX vers tableau pixels du font
	//memcpy(font->pixels, tmp->pixels, font->nboctets); // 3 channels
	// 1 channel
	for(int y=0; y<font->hauteur_caractere; y++)
		for (int x=0; x<font->width; x++)
		{
			u8* p = tmp->pixels  + (x+y*tmp->width)*tmp->nchannels;
			font->pixels[x+y*font->width] = p[0];
		}
	pix_destroy(tmp);
	return font;
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Crée un PIXFONT à partir d'un fichier .font 1 channel
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
PIXFONT *pix_load_font(char *fontname)
{
	PIXFONT *font;
	FILE *pF;

	pF = fopen(fontname, "rb");
	if (!pF) return NULL;
	font = (PIXFONT*) malloc(sizeof(PIXFONT));
	// lire header
	fread(&font->width            , sizeof(int), 1, pF);
	fread(&font->taille           , sizeof(int), 1, pF);
	fread(&font->largeur_caractere, sizeof(int), 1, pF);
	fread(&font->hauteur_caractere, sizeof(int), 1, pF);
	// lire pixels
	long nboctets = font->width * font->hauteur_caractere;
	font->pixels = (u8*) malloc(nboctets);
	fread(font->pixels, 1, nboctets, pF);
	fclose(pF);
	return font;
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Enregistrer un font sur le disque. Après création ou modification
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_save_font(PIXFONT *font, char *filename)
{
	FILE *pF = fopen(filename, "wb");
	fwrite(&font->width            , 1, sizeof(int), pF);
	fwrite(&font->taille           , 1, sizeof(int), pF);
	fwrite(&font->largeur_caractere, 1, sizeof(int), pF);
	fwrite(&font->hauteur_caractere, 1, sizeof(int), pF);
	fwrite(font->pixels, 1, font->width*font->hauteur_caractere, pF);
	fclose(pF);
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Ecrire du texte sur une image.
  Pour chaque caractère à écrire, parcourt pixels du carreau concerné sur le
  font et le transfère pixels sur image.
  Ex. le 'a' il est ou? c'est l' ASCII 65 donc le carreau qui commence
  au 65-32e octet sur le font (les 32 premiers n'ont pas été inclus car
  non printable, pour gagner de la place).
  Partant de cet octet il n'y a qu'à transférer + blitting
    font->largeur_caracteres * font->hauteur_caracteres
  octets. Ces données sont dans le header de chaque pixfont stocké sur le disque.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_draw_text(PIX *img, int x1, int y1, char *text, RGBcolor forecolor, PIXFONT *font)
{
	int offset=x1;
	int top = img->height - font->hauteur_caractere - y1;

	for (int n=0; n<strlen(text); n++)
	{
		// Cas particulier: retour à la ligne! le seul des 32 premiers qui n'est pas ignoré
		if (text[n]=='\n')
		{
			top -= font->hauteur_caractere;
			offset=x1;
			continue;
		}
		// Transfère pixel font vers pixel image: boucle sur le carreau concerné du font
		for (int x=0; x<font->largeur_caractere; x++)
		{
			for(int y=0; y<font->hauteur_caractere; y++)
			{
				// pixel du font à transférer
				u8 *p1 = font->pixels + (x+((u8)text[n]-32)*font->largeur_caractere + y*font->width);
				// sur ce pixel image
				u8 *p2 = img->pixels  + (x+offset + ((top+y) * img->width) )*img->nchannels;

				if (*p1!=255) // ignore le fond blanc du font
				{
					if (*p1==0) // si le pixel du font est noir, le met direct
					{
						p2[0] = forecolor.r;
						p2[1] = forecolor.g;
						p2[2] = forecolor.b;
					}
					else // sinon le blende avec pixel image
					{
						float f1 = (float)*p1/255.f;
						float f2 = 1 - f1;
						p2[0]=(u8)(forecolor.r*f2 + p2[0]*f1);
						p2[1]=(u8)(forecolor.g*f2 + p2[1]*f1);
						p2[2]=(u8)(forecolor.b*f2 + p2[2]*f1);
					}
				}
			}
		}
		offset += font->largeur_caractere;
		// attention si offset dépasse le bord de l'image ne plus rien faire!
		if (offset > img->width) return;
		// sinon passer au char suivant
	}
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Ecrire du texte sur une image avec couleur d'arrière plan transluicide.
  Mettre 1 en alpha pour avoir un fond opaque.
  Utile pour la lisibilité du texte quel que soit l'arrière plan.
  + de calculs que draw_text() donc + lent d'ou 2e fonction à part
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_draw_text_alpha(PIX *img, int x1, int y1, char *text, RGBcolor forecolor, PIXFONT *font, RGBcolor backcolor, float alpha)
{
  int x,y, n=0, top, offset=x1;
  u8 *p1, *p2;
  float f1, f2;

  top = img->height - font->hauteur_caractere - y1;

  for (n=0; n<strlen(text); n++)
  {
    // Cas particulier: retour à la ligne! le seul des 32 premiers qui n'est pas ignoré
    if (text[n]=='\n')
    {
      top -= font->hauteur_caractere;
      //hey si top est plus bas que l'image ne rien faire!
      if (top > img->height) return;
      // repositionne pixel "écriture" à gauche sur le carreau
      offset=x1;
      continue;
    }
    // Transfère pixel font vers pixel image: boucle sur le carreau concerné du font
    for (x=0; x<font->largeur_caractere; x++)
    {
      for(y=0; y<font->hauteur_caractere; y++)
      {
        // pixel du font à transférer
        p1 = font->pixels + (x+((u8)text[n]-32)*font->largeur_caractere + y*font->width);
        // sur ce pixel image
        p2 = img->pixels  + (x+offset + ((top+y) * img->width) )*img->nchannels;

        if (*p1==255) // blanc du font devient backcolor*alpha
        {
          f1 = 1 - alpha;
          p2[0]=(u8)(backcolor.r*alpha + p2[0]*f1);
          p2[1]=(u8)(backcolor.g*alpha + p2[1]*f1);
          p2[2]=(u8)(backcolor.b*alpha + p2[2]*f1);
        }
        else
        {
          if (*p1==0) // si le pixel du font est noir, le met direct
          {
            p2[0] = forecolor.r;
            p2[1] = forecolor.g;
            p2[2] = forecolor.b;
          }
          else // sinon le blende avec pixel image lui-même blendé avec backcolor
          {
            f1 = 1 - alpha;
            p2[0]=(u8)(backcolor.r*alpha + p2[0]*f1);
            p2[1]=(u8)(backcolor.g*alpha + p2[1]*f1);
            p2[2]=(u8)(backcolor.b*alpha + p2[2]*f1);

            f1 = (float)*p1/255.f;
            f2 = 1 - f1;
            p2[0]=(u8)(forecolor.r*f2 + p2[0]*f1);
            p2[1]=(u8)(forecolor.g*f2 + p2[1]*f1);
            p2[2]=(u8)(forecolor.b*f2 + p2[2]*f1);
          }
        }
      }
    }
    // repositionne sur carreau suivant sur l'image
    offset += font->largeur_caractere;
    // attention si offset dépasse le bord de l'image ne plus rien faire!
    if (offset > img->width) return;
    // sinon passer au char suivant
  }
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Détruire un font: libérer mémoire
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_destroy_font (PIXFONT *font)
{
	if(font->pixels) free(font->pixels);
	if(font) free(font);
}

