/* ___ _      _ _ _     
  / _ (_)_  _| (_) |__  
 / /_)/ \ \/ / | | '_ \ 
/ ___/| |>  <| | | |_) |
\/    |_/_/\_\_|_|_.__/ */
#include <pixlib.h>

typedef struct
{
  unsigned short bfType; //must always be set to 'BM' to declare that this is a .bmp-file
  unsigned int   bfSize; // size of the file in bytes
  unsigned short bfReserved1; //must always be set to zero
  unsigned short bfReserved2; //must always be set to zero
  unsigned int   bfOffBits; //offset from the beginning of the file to the bitmap data
} bmpFHEAD;

typedef struct //bmpIHEAD
{
  unsigned int   biSize; // size of the BITMAPINFOHEADER structure, in bytes
  int            biWidth; // of the image, in pixels
  int            biHeight;
  unsigned short biPlanes; //number of planes of the target device, must be set to zero
  unsigned short biBitCount; // number of bits per pixel
  unsigned int   biCompression; //usually set to zero (no compression)
  unsigned int   biSizeImage; //size of the image data, in bytes. 0 if no compression
  int            biXPelsPerMeter; //horizontal pixels per meter on the designated targer device, usually set to zero.
  int            biYPelsPerMeter; 
  unsigned int   biClrUsed; //number of colors used in the bitmap, if set to zero the number of colors is calculated using the biBitCount member
  unsigned int   biClrImportant; //if set to zero, all colors are important
} bmpIHEAD;

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Créer un pixbuf depuis un BITMAP sur le disque
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
PIX *pix_load_bmp(const char *filename)
{
  PIX *bmp=NULL;
  int x, y, a;
  bmpFHEAD h1;
  bmpIHEAD h2;
  RGBpixel p;
  char swp;
	
  FILE *pF = fopen(filename, "rb");
  if (!pF) {printf("Fichier introuvable [%s]\n", filename); return NULL;}

  //FILEHEADER
  fread(&h1.bfType     , sizeof(h1.bfType)     ,1 , pF);
  fread(&h1.bfSize     , sizeof(h1.bfSize)     ,1 , pF);
  fread(&h1.bfReserved1, sizeof(h1.bfReserved1),1 , pF);
  fread(&h1.bfReserved2, sizeof(h1.bfReserved2),1 , pF);
  fread(&h1.bfOffBits  , sizeof(h1.bfOffBits)  ,1 , pF);

  //INFOHEADER
  fread(&h2, sizeof(h2), 1, pF);

  bmp = pix_create(h2.biWidth, h2.biHeight, 3);//(u8)(h2.biBitCount/8));
  if (!bmp) {  fclose(pF); return NULL;}

  // PIXELS
  for(y=0; y < bmp->height; y++)
  {
    for(x=0; x < bmp->width; x++)
    {
      fread (&p, sizeof(p), 1, pF);
      pix_setPixel (bmp, x, y, p.b, p.g, p.r);

    }
    //Add padding bytes FIX: Move filepointer instead
    for (a=0; a<(4-((3*bmp->width)%4))%4; a++)
			fread (&swp, sizeof(char), 1, pF);
  }
  fclose(pF);
  return bmp;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Enregistrer un pixbuf sur le disque au format BITMAP
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
int pix_save_bmp (PIX *bmp, const char *filename)
{
	bmpFHEAD filehead; //File Header
	bmpIHEAD infohead; //Info Header
	u8 swp;
	long bitsize;
	int x, y, a;
	RGBpixel p;
	FILE *pF;
  
	pF = fopen(filename, "wb");
	if (!bmp->pixels || !pF) //If there's no pixels nor a file, don't even start writing
	{
		fclose(pF);
		return 0;
	}

	bitsize  = bmp->width * bmp->height * 3;
	bitsize += bitsize%4;	 //and round to next 4 bytes (see byte alignement in compiler options!)

	//FILEHEADER
	filehead.bfType = 'MB';//0x4d42;//'MB';//0x4d42;
	filehead.bfSize = sizeof(bmpFHEAD)-2 + sizeof(bmpIHEAD) + bitsize;	// -2 due to byte alignment
	filehead.bfReserved1 = 0;
	filehead.bfReserved2 = 0;
	filehead.bfOffBits   = sizeof(bmpFHEAD)-2 + sizeof(bmpIHEAD);		// -2 due to byte alignment

	fwrite(&filehead.bfType,	sizeof(filehead.bfType), 1, pF);
	fwrite(&filehead.bfSize,	sizeof(filehead.bfSize), 1, pF);
	fwrite(&filehead.bfReserved1,	sizeof(filehead.bfReserved1), 1, pF);
	fwrite(&filehead.bfReserved2,	sizeof(filehead.bfReserved2), 1, pF);
	fwrite(&filehead.bfOffBits, sizeof(filehead.bfOffBits),	1, pF);

	//HEADER
	infohead.biSize          = sizeof (bmpIHEAD);
	infohead.biWidth         = bmp->width;
	infohead.biHeight        = bmp->height;
	infohead.biPlanes        = 1;
	infohead.biBitCount      = bmp->nchannels*8; // bits par pixel : nbchannels*8
	infohead.biCompression   = 0;  //RGB
	infohead.biSizeImage     = bitsize;
	infohead.biXPelsPerMeter = 2800;
	infohead.biYPelsPerMeter = 2800;
	infohead.biClrUsed       = 0;
	infohead.biClrImportant  = 0;

	fwrite(&infohead, sizeof (infohead), 1, pF);

	//BITMAP
	bitsize=0;
	for(y=0; y<bmp->height; y++) 
	{
		for(x=0; x<bmp->width; x++) 
		{
			bitsize = (x+(y*bmp->width))*bmp->nchannels;//3;
			p.r = bmp->pixels[bitsize+2];
			p.g = bmp->pixels[bitsize+1];
			p.b = bmp->pixels[bitsize+0];
			fwrite (&p, sizeof (p), 1, pF);
 		}

		//Add padding bytes
		swp=0;
 		for (a=0; a<(4-((bmp->nchannels*bmp->width)%4))%4; a++)
			fwrite (&swp, sizeof(char), 1, pF);
	}

	fclose (pF);
	return 1;
}

/*
int pix_save_bmp (PIX *bmp, const char *filename)
{
	FILE *fp;
	u8 bmpfileheader[14] = {'B','M', 0,0,0,0, 0,0, 0,0, 54,0,0,0};
	u8 bmpinfoheader[40] = {40,0,0,0, 0,0,0,0, 0,0,0,0, 1,0, 24,0};
	int nbpixels = bmp->nchannels * bmp->width * bmp->height;
	int filesize = 54 + nbpixels;

	bmpfileheader[ 2] = (unsigned char)(filesize    );
	bmpfileheader[ 3] = (unsigned char)(filesize>> 8);
	bmpfileheader[ 4] = (unsigned char)(filesize>>16);
	bmpfileheader[ 5] = (unsigned char)(filesize>>24);

	bmpinfoheader[ 4] = (unsigned char)(       bmp->width    );
	bmpinfoheader[ 5] = (unsigned char)(       bmp->width>> 8);
	bmpinfoheader[ 6] = (unsigned char)(       bmp->width>>16);
	bmpinfoheader[ 7] = (unsigned char)(       bmp->width>>24);
	bmpinfoheader[ 8] = (unsigned char)(       bmp->height    );
	bmpinfoheader[ 9] = (unsigned char)(       bmp->height>> 8);
	bmpinfoheader[10] = (unsigned char)(       bmp->height>>16);
	bmpinfoheader[11] = (unsigned char)(       bmp->height>>24);

	fp = fopen(filename,"wb");
	if (!bmp->pixels || !fp) return 0;

	fwrite(bmpfileheader,1,14,fp);
	fwrite(bmpinfoheader,1,40,fp);

	/ *for(int i=0; i<bmp->height; i++)
	{
		fwrite(bmp->pixels + (bmp->width*(bmp->height-i-1)*3),3,bmp->width,fp);
		u8 bmppad[3] = {0,0,0};
		fwrite(bmppad,1,(4-(bmp->width*bmp->nchannels)%4)%4,fp);
	}* /

	for(int y=0; y<bmp->height; y++) {
		for(int x=0; x<bmp->width; x++) {
			long bitsize = (x+(y*bmp->width))*bmp->nchannels;
			RGBpixel p;
			p.r = bmp->pixels[bitsize+2];
			p.g = bmp->pixels[bitsize+1];
			p.b = bmp->pixels[bitsize+0];
			fwrite (&p, sizeof (p), 1, fp);
		}
		//u8 bmppad[3] = {0,0,0};
		//fwrite(bmppad,1,(4-(bmp->width*bmp->nchannels)%4)%4,fp);

		//Add padding bytes
		//u8 swp=0;
		//for (int a=0; a<(4-((bmp->nchannels*bmp->width)%4))%4; a++)
		  //fwrite (&swp, sizeof(char), 1, fp);
	}

	fclose(fp);
	return 1;
}*/
