/* ___ _      _ _ _     
  / _ (_)_  _| (_) |__  
 / /_)/ \ \/ / | | '_ \ 
/ ___/| |>  <| | | |_) |
\/    |_/_/\_\_|_|_.__/ */
#include <pixlib.h>
#include <jpeglib.h>    
#include <jerror.h>

// http://gnuwin32.sourceforge.net/packages/jpeg.htm
//#include "../libjpeg/jpeglib.h"
//#include "../libjpeg/jerror.h"

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Charger une image JPG du disque dans un PIX
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
PIX *pix_load_jpg (const char *filename)
{
	struct jpeg_decompress_struct cinfo;
	struct jpeg_error_mgr jerr;
	FILE *pF;
	unsigned char *ligne;
	PIX *img;

	cinfo.err = jpeg_std_error(&jerr);
	jpeg_create_decompress(&cinfo);

	pF = fopen(filename,"rb");
	if (!pF) {printf("Fichier introuvable [%s]\n", filename); return NULL;}

	jpeg_stdio_src(&cinfo, pF);
	jpeg_read_header(&cinfo, TRUE);

	if (cinfo.jpeg_color_space==JCS_GRAYSCALE)
	{
		printf("Erreur : l'image doit etre de type RGB. ici JCS_GRAYSCALE\n");
		return NULL;
	}

	jpeg_start_decompress(&cinfo);

	img = pix_create(cinfo.image_width, cinfo.image_height, cinfo.out_color_components);

	ligne = img->pixels;
	while (cinfo.output_scanline < cinfo.output_height)
	{
		ligne = img->pixels + cinfo.out_color_components * img->width * cinfo.output_scanline;
		jpeg_read_scanlines(&cinfo,&ligne,1);
	}

	jpeg_finish_decompress(&cinfo);
	jpeg_destroy_decompress(&cinfo);
	fclose(pF);

	pix_flip_vertical(img);
	return img;
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Enregistrer un PIX dans une image JPG sur le disque
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
int pix_save_jpg (PIX *img, const char * filename, int quality)
{
	struct jpeg_compress_struct cinfo;
	struct jpeg_error_mgr jerr;
	FILE *pF;
	int row_stride;
	JSAMPROW row_pointer[1];

	pF = fopen(filename, "wb");
	if (pF == NULL) return 0;

	cinfo.err = jpeg_std_error(&jerr);

	jpeg_create_compress(&cinfo);
	jpeg_stdio_dest(&cinfo, pF);

	cinfo.image_width = img->width;
	cinfo.image_height = img->height;
	cinfo.input_components = 3;

	cinfo.in_color_space = JCS_RGB;

	jpeg_set_defaults(&cinfo);
	if     (quality < 0)   quality = 0;
	else if(quality > 100) quality = 100;
	jpeg_set_quality(&cinfo, quality, TRUE);

	pix_flip_vertical(img);

	jpeg_start_compress(&cinfo, TRUE);
	row_stride = img->width * img->nchannels;//cinfo.input_components;

	while (cinfo.next_scanline < cinfo.image_height)
	{
		row_pointer[0] = &img->pixels[cinfo.next_scanline * row_stride];
		(void)jpeg_write_scanlines(&cinfo, row_pointer, 1);
	}

	jpeg_finish_compress(&cinfo);
	fclose(pF);

	jpeg_destroy_compress(&cinfo);
	return 1;
}

