/* ___ _      _ _ _     
  / _ (_)_  _| (_) |__  
 / /_)/ \ \/ / | | '_ \ 
/ ___/| |>  <| | | |_) |
\/    |_/_/\_\_|_|_.__/ */
#include <pixlib.h>
#include <stdlib.h>

/*	PGM (Portable GreyMap) est une image 1 channel au format ASCII.
	http://paulbourke.net/dataformats/ppm/

	Exemple de fichier PGM:
P2
3 2
255
10 20 30 
40 50 60 

	La 1re ligne est le type : P2
	2e ligne: width height
	3e ligne: coulmax. C'est 255 si les valeurs oscillent entre 0 et 255 mais ça peut être plus.
	Puis chaque ligne de pixels (en partant du coin supérieur gauche) séparée par \n.
	La valeur de chaque pixel est suivie d'un espace; le dernier caractère du fichier est un espace.
*/
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Créer un pixbuf depuis un PGM sur le disque
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
PIX *pix_load_pgm(const char *filename)
{
	// Ouvre le fichier ASCII et place son contenu dans un string
	FILE *fp = fopen (filename, "r");
	if (fp==NULL) {printf("Can't find %s\n", filename); return NULL;}
	fseek (fp, 0, SEEK_END);
	long lSize = ftell(fp);
	rewind (fp);
	char *s = (char*) malloc(lSize);
	if (s == NULL) {printf("Can't malloc %ld\n", lSize); return NULL;}
	fread (s,1,lSize,fp);
	fclose (fp);

	// parcourt le string ligne par ligne

	// la 1re ligne n'est pas "P2\n"
	if(s[0]!= 'P' && s[1]!= '2' && s[2]!= '\n'){puts("Erreur: Header non P2"); return NULL;}

	int n = 3;
	char mot[17];// width height à 8 chiffres: 54.125.447 X 65.475.852 pixels
	u8 b = 0;
	while(s[n] != ' '){mot[b++] = s[n++];}
	mot[b]=0;b=0;
	int width = atoi(mot);

	while(s[n] != '\n'){mot[b++] = s[n++];}
	mot[b]=0;b=0;n++;
	int height = atoi(mot);

	while(s[n] != '\n'){mot[b++] = s[n++];}
	mot[b]=0;b=0;n++;
	int coulmax = atoi(mot);
	printf("%s est un PGM %dX%d pixels max:%d\n", filename, width, height, coulmax);
	if(coulmax != 255) { printf("%s image P2 non 255 (%d), que faire?\n", filename, coulmax); return NULL;}

	PIX *img = pix_create(width, height, 1);
	// les lignes de pixels
	int y = 0;
	int start = n;
	n = 0;
	int i = 0;
	while(n < lSize){
		u8 c = s[start+n]; 
		if(c==' '){
			mot[b]=0;b=0;
			img->pixels[i] = atoi(mot);
			//printf("[%s] ", mot);
			//printf("%d ", img->pixels[i]);
			i++;
		}
		else if (c=='\n') y++; //puts("");}
		else mot[b++] = c;
		n++;
	}

	free(s);
	return img;
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Enregistrer un pixbuf sur le disque au format PGM
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
int pix_save_pgm (PIX *img, const char *filename)
{
	if(img->nchannels != 1) {puts("img->nchannels != 1"); return 0;}

	FILE *fp = fopen(filename,"w");
	fprintf(fp, "P2\n%d %d\n255\n", img->width, img->height);
	int n = 0;
	int y = 0;
	int len = img->width * img->height;
	while(n < len){
		fprintf(fp,"%d ",img->pixels[n]);
		//printf("\t%d ",img->pixels[n]);
		n++;

		y++;
		if(y==img->width){
			fprintf(fp,"\n");
			//puts("");
			y = 0;
		}
	}
	fclose(fp);
	return 1;
}


/* j'ai trouvé ça dans la tourelle caméra https://www.masta.fr/3D/projets/tourelle/camera.h.html
// image gris 1 channel P5:binaire P2:ASCII
// 228Ko ppm -> 76Ko pgm intéressant pour le réseau, ça permettrait d'avoir une résolution plus grande ou plus de FPS
static void save_PGM(u32 i, u32 x_res, u32 y_res, u8 *data, size_t data_length) {
    FILE *fout;
    char out_name[9];

    sprintf(out_name, "%03d.pgm", i);
    fout = fopen(out_name, "w");
    if (!fout) print_error(WRITE_PPM_ERR, errno);
    fprintf(fout, "P5\n%d %d 255\n", x_res, y_res);
    fwrite(data, data_length, 1, fout);
    fclose(fout);
}
*/


