/* ___ _      _ _ _     
  / _ (_)_  _| (_) |__  
 / /_)/ \ \/ / | | '_ \ 
/ ___/| |>  <| | | |_) |
\/    |_/_/\_\_|_|_.__/ */
#include <pixlib.h>
#include <stdlib.h>

/*	PPM (Portable PixMap) est une image 3 channels au format ASCII.
	http://paulbourke.net/dataformats/ppm/

	Exemple de fichier PPM:
P3
# example from the man page
4 4
15
 0  0  0    0  0  0    0  0  0   15  0 15
 0  0  0    0 15  7    0  0  0    0  0  0
 0  0  0    0  0  0    0 15  7    0  0  0
15  0 15    0  0  0    0  0  0    0  0  0

	La 1re ligne est le type : P3
	2e ligne: width height
	3e ligne: coulmax. C'est 255 si les valeurs oscillent entre 0 et 255 mais ça peut être plus.
	Puis chaque ligne de pixels (en partant du coin supérieur gauche) séparée par \n.
	La valeur RG ou B de chaque pixel est suivie d'un espace; le dernier caractère du fichier n'est pas forcément est un espace.
*/
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Créer un pixbuf depuis un PPM sur le disque
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
PIX *pix_load_ppm(const char *filename)
{
	// Ouvre le fichier ASCII et place son contenu dans un string
	FILE *fp = fopen (filename, "r");
	if (fp==NULL) {printf("Can't find %s\n", filename); return NULL;}
	fseek (fp, 0, SEEK_END);
	long lSize = ftell(fp);
	rewind (fp);
	char *s = (char*) malloc(lSize);
	if (s == NULL) {printf("Can't malloc %ld\n", lSize); return NULL;}
	fread (s,1,lSize,fp);
	fclose (fp);

	// parcourt le string ligne par ligne

	// la 1re ligne n'est pas "P2\n"
	if(s[0]!= 'P' && s[1]!= '3' && s[2]!= '\n'){puts("Erreur: Header non P3"); return NULL;}

	int n = 3;
	char mot[17];// width height à 8 chiffres: 54.125.447 X 65.475.852 pixels
	u8 b = 0;
	while(s[n] != ' '){mot[b++] = s[n++];}
	mot[b]=0;b=0;
	int width = atoi(mot);

	while(s[n] != '\n'){mot[b++] = s[n++];}
	mot[b]=0;b=0;n++;
	int height = atoi(mot);

	while(s[n] != '\n'){mot[b++] = s[n++];}
	mot[b]=0;b=0;n++;
	int coulmax = atoi(mot);
	printf("%s est un PPM %dX%d pixels max:%d\n", filename, width, height, coulmax);
	if(coulmax != 255) { printf("%s image P3 non 255 (%d), que faire?\n", filename, coulmax); return NULL;}

	PIX *img = pix_create(width, height, 3);
	// les lignes de pixels
	int y = 0;
	int start = n;
	n = 0;
	int i = 0;
	while(n < lSize){
		u8 c = s[start+n]; 
		if(c==' '){
			if(b!=0){
				mot[b]=0;b=0;
				img->pixels[i] = atoi(mot);
				//printf("[%s] ", mot);
				printf("%d ", img->pixels[i]);
				i++;
			}
		}
		else if (c=='\n') {y++; puts("");}
		else mot[b++] = c;
		n++;
	}

	free(s);
	return img;
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Enregistrer un pixbuf sur le disque au format PPM
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
int pix_save_ppm (PIX *img, const char *filename)
{
	FILE *fp = fopen(filename,"w");
	fprintf(fp, "P3\n%d %d\n255\n", img->width, img->height);

	for(int y=0; y<img->height; y++){
		for(int x=0; x<img->width; x++){
			u8 *p = img->pixels + (x + y * img->width) * img->nchannels;
			fprintf(fp,"%d %d %d\t",p[0],p[1],p[2]);
			//printf("%d %d %d\t",p[0],p[1],p[2]);
		}	
		fprintf(fp,"\n");
		//puts("");
	}

	fclose(fp);
	return 1;
}

/* j'ai trouvé ça dans la tourelle caméra https://www.masta.fr/3D/projets/tourelle/camera.h.html
/ image couleur 3 channels
static void save_ppm(u32 i, u32 x_res, u32 y_res, char *data, size_t data_length) {
    FILE *fout;
    char out_name[9];

    sprintf(out_name, "%03d.ppm", i);
    fout = fopen(out_name, "w");
    if (!fout) print_error(WRITE_PPM_ERR, errno);
    fprintf(fout, "P6\n%d %d 255\n", x_res, y_res);
    fwrite(data, data_length, 1, fout);
    fclose(fout);
}
*/

