/* ___ _      _ _ _     
  / _ (_)_  _| (_) |__  
 / /_)/ \ \/ / | | '_ \ 
/ ___/| |>  <| | | |_) |
\/    |_/_/\_\_|_|_.__/ */
// le format RAW n'a pas de header: l'image est forcément carrée
#include <pixlib.h>
#include <math.h>

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Créer un pixbuf à n channels depuis une image RAW sur le disque
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
PIX *pix_load_raw(const char *filename, unsigned char nchannels)
{
	PIX *img=NULL;
	long nboctets;
	int taille;
	
	FILE *pF = fopen(filename, "rb");
	if (!pF) {printf("Fichier introuvable [%s]\n", filename); return NULL;}

	fseek(pF, 0, SEEK_END);
	nboctets = ftell(pF);
	rewind(pF);

	// le RAW n'a pas de header: l'image est forcément carrée
	taille = sqrt(nboctets);
	img = pix_create(taille, taille, nchannels);
	if (!img) {fclose(pF); return NULL;}

	fread (img->pixels, 1, img->width*img->height, pF);

	fclose(pF);
	return img;
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Enregistrer un pixbuf au format RAW sur le disque
  Le nombre de channel détermine la taille du raw
  Retourne 1 si succès, 0 si échec
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
int pix_save_raw (PIX *img, const char *filename, unsigned char nchannels)
{
	//unsigned char swp;
	long taille=0, nboctets;
	FILE *pF = fopen(filename, "wb");
	if (!pF) return 0;

	nboctets = img->width * img->height * nchannels;
	taille = fwrite (img->pixels, 1, nboctets, pF);
	fclose (pF);

	if (taille != nboctets)  return 0;
	//printf("taille ecrite=%ld nboctets a ecrire=%ld\n", taille,nboctets);
	return 1;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Créer un pixbuf depuis une image RAW 3 channels sur le disque
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*
PIX *pix_load_raw_3channels(const char *filename)
{
  PIX *img=NULL;
  long nboctets;
  int x, y, a;
  pixel p;
  char swp;
	
  FILE *pF = fopen(filename, "rb");
  if (!pF) return NULL;

  fseek(pF, 0, SEEK_END);
  nboctets = ftell(pF);
  rewind(pF);

  img = pix_create(nboctets, nboctets, 3);
  if (!img) {fclose(pF); return NULL;}

  // PIXELS
  for(y=0; y < img->height; y++)
  {
    for(x=0; x < img->width; x++)
    {
      fread (&p, sizeof(p), 1, pF);
      pix_setcolor (img, x, y, p.b, p.g, p.r);
    }
  }
  fclose(pF);
  return img;
}*/

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Enregistrer un pixbuf au format RAW 3 channels sur le disque
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*int pix_save_raw_3channels (PIX *img, const char *filename)
{
  unsigned char swp;
  long a;
  int x, y;
  pixel p;
  FILE *pF;

  pF = fopen(filename, "wb");

  if (!img->pixels || !pF)
  {
    fclose (pF);
    return 0;
  }

  for(y=0; y<img->height; y++) 
  {
    for(x=0; x<img->width; x++) 
    {
      a = (x+(y*img->width))*3;
      p.r = img->pixels[a+2];
      p.g = img->pixels[a+1];
      p.b = img->pixels[a+0];
      fwrite (&p, sizeof (p), 1, pF);
    }
  }

  fclose (pF);
  return 1;
}*/

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Créer un pixbuf depuis une image RAW 1 channels sur le disque
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*PIX *pix_load_raw_1channel(const char *filename)
{
  PIX *img=NULL;
  long nboctets;
  int x, y, a;
  pixel p;
  char c;
	
  FILE *pF = fopen(filename, "rb");
  if (!pF) return NULL;

  fseek(pF, 0, SEEK_END);
  nboctets = ftell(pF);
  rewind(pF);

  img = pix_create(nboctets, nboctets, 1);
  if (!img) {fclose(pF); return NULL;}

  // PIXELS
  for(y=0; y < img->height; y++)
  {
    for(x=0; x < img->width; x++)
    {
      fread (&c, sizeof(char), 1, pF);
      pix_setcolor (img, x,y, c,c,c);
    }
  }
  fclose(pF);
  return img;
}*/
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Enregistrer un pixbuf au format RAW 1 channel sur le disque
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*int pix_save_raw_1channel (PIX *img, const char *filename)
{
  unsigned char swp;
  int x, y;
  FILE *pF;

  pF = fopen(filename, "wb");

  if (!img->pixels || !pF)
  {
    fclose (pF);
    return 0;
  }

  fwrite (&img->pixels, img->width*img->height*sizeof(char), 1, pF);

  fclose (pF);
  return 1;
}*/
