/* ___ _      _ _ _     
  / _ (_)_  _| (_) |__  
 / /_)/ \ \/ / | | '_ \ 
/ ___/| |>  <| | | |_) |
\/    |_/_/\_\_|_|_.__/ */
/* tga */
#include <pixlib.h>

#define	TGA_ERROR_FILE_OPEN				-5
#define TGA_ERROR_READING_FILE			-4
#define TGA_ERROR_INDEXED_COLOR			-3
#define TGA_ERROR_MEMORY				-2
#define TGA_ERROR_COMPRESSED_FILE		-1
#define TGA_OK							 0

#define PIXLIB_API _declspec (dllexport)

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// this is the function to call when we want to load an image
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
PIX* pix_load_tga(const char *filename)
{
	FILE *fp;
    PIX *tga = NULL;
	int i,total;
	unsigned char aux;
	unsigned char cGarbage;
	short int iGarbage;
	short int width, height;

    // open the file for reading (binary mode)
	fp = fopen(filename, "rb");
	if (!fp) {printf("Fichier introuvable [%s]\n", filename); return NULL;}

    // read header
	fread(&cGarbage, sizeof(unsigned char), 1, fp);
	fread(&cGarbage, sizeof(unsigned char), 1, fp);

    // type must be 2 or 3
    unsigned char type;
	// 1	-	colour map image
	// 2	-	RGB(A) uncompressed
	// 3	-	greyscale uncompressed
	// 9	-	greyscale RLE (compressed)
	// 10	-	RGB(A) RLE (compressed)
	fread(&type, sizeof(unsigned char), 1, fp);

    printf("tga type : %d\n", type);

	fread(&iGarbage, sizeof(short int), 1, fp);
	fread(&iGarbage, sizeof(short int), 1, fp);
	fread(&cGarbage, sizeof(unsigned char), 1, fp);
	fread(&iGarbage, sizeof(short int), 1, fp);
	fread(&iGarbage, sizeof(short int), 1, fp);

	fread(&width, sizeof(short int), 1, fp);
	fread(&height, sizeof(short int), 1, fp);

	unsigned char pixelDepth;
    // 8 - The image is greyscale, 1 channel
    // 24 - The image is RGB, each pixel requires 3 components of the array
    // 32 - An RGBA image, where each pixel requires 4 components
	fread(&pixelDepth, sizeof(unsigned char), 1, fp);
    //printf("pixeldepth:%d\n", pixelDepth);

    tga = pix_create(width, height, (unsigned char)(pixelDepth/8));

	fread(&cGarbage, sizeof(unsigned char), 1, fp);
/*
    // check for errors when loading the header
	if (ferror(file)) {
		info->status = TGA_ERROR_READING_FILE;
		fclose(file);
		return(info);
	}

    // check if the image is color indexed
	if (info->type == 1) {
		info->status = TGA_ERROR_INDEXED_COLOR;
		fclose(file);
		return(info);
	}
    // check for other types (compressed images)
	if ((info->type != 2) && (info->type !=3)) {
		info->status = TGA_ERROR_COMPRESSED_FILE;
		fclose(file);
		return(info);
	}
*/
    // total is the number of bytes to read
	total = tga->height * tga->width * tga->nchannels;

    // allocate memory for image pixels
	//tga->pixels = (unsigned char *)malloc(sizeof(unsigned char) * total);

    // check to make sure we have the memory required
    if (!tga) {  fclose(fp); return NULL;}

    // finally load the image pixels
	fread(tga->pixels,sizeof(unsigned char),total,fp);

    // mode=3 or 4 implies that the image is RGB(A). However TGA
    // stores it as BGR(A) so we'll have to swap R and B.
	if (tga->nchannels >= 3)
		for (i=0; i < total; i+= tga->nchannels)
        {
			aux = tga->pixels[i];
			tga->pixels[i] = tga->pixels[i+2];
			tga->pixels[i+2] = aux;
		}

	fclose(fp);

	//info->status = TGA_OK;
	return tga;
}		
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// converts RGB to greyscale
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*
void tgaRGBtoGreyscale(PIX* tga)
{
	int i,j;

	unsigned char *newImageData;

    // if the image is already greyscale do nothing
	if (tga->nchannels == 1) return;

    // compute the number of actual components
	//mode = info->pixelDepth / 8;

    // allocate an array for the new image data
	newImageData = (unsigned char *)malloc(sizeof(unsigned char) * 
											tga->height * tga->width);
	if (newImageData == NULL) return;

    // convert pixels: greyscale = o.30 * R + 0.59 * G + 0.11 * B
	for (i = 0,j = 0; j < tga->width * tga->height; i +=tga->nchannels, j++)
		newImageData[j] =	(unsigned char)(
                               0.30 * tga->pixels[i] +     
						       0.59 * tga->pixels[i+1] +
						       0.11 * tga->pixels[i+2]);
    //free old image data
	free(tga->pixels);

    // reassign pixelDepth and type according to the new image type
	tga->nchannels = 1;
	//info->type = 3;
     // reassing imageData to the new array.
	tga->pixels = newImageData;
}
/ *%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   saves an array of pixels as a TGA image
   
   Apparently there's no RLE compression so the size is the same as a bmp
   and it can read only non RLE encoded files :(

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*
_declspec (dllexport) int pix_save_tga(PIX *tga, const char *filename)
{
	unsigned char cGarbage = 0, type,mode,aux;
	short int iGarbage = 0;
	int i;
	FILE *file;

    // open file and check for errors
	file = fopen(filename, "wb");
	if (file == NULL) {
		return(TGA_ERROR_FILE_OPEN);
	}

    // compute image type: 2 for RGB(A), 3 for greyscale
	if ((tga->nchannels == 3) || (tga->nchannels == 4))
		type = 2;
	else
		type = 3;

    // write the header
	fwrite(&cGarbage, sizeof(unsigned char), 1, file);
	fwrite(&cGarbage, sizeof(unsigned char), 1, file);

	fwrite(&type, sizeof(unsigned char), 1, file);

	fwrite(&iGarbage, sizeof(short int), 1, file);
	fwrite(&iGarbage, sizeof(short int), 1, file);
	fwrite(&cGarbage, sizeof(unsigned char), 1, file);
	fwrite(&iGarbage, sizeof(short int), 1, file);
	fwrite(&iGarbage, sizeof(short int), 1, file);

    short int height = (short int)tga->height;
    short int width = (short int)tga->width;

	fwrite(&width, sizeof(short int), 1, file);
	fwrite(&height, sizeof(short int), 1, file);

    unsigned char pixelDepth = tga->nchannels * 8;
	fwrite(&pixelDepth, sizeof(unsigned char), 1, file);

	fwrite(&cGarbage, sizeof(unsigned char), 1, file);

    // convert the image data from RGB(a) to BGR(A)
	if (mode >= 3)
	for (i=0; i < tga->width * tga->height * tga->nchannels ; i+= tga->nchannels)
    {
		aux = tga->pixels[i];
		tga->pixels[i] = tga->pixels[i+2];
		tga->pixels[i+2] = aux;
	}

    // save the image data
	fwrite(tga->pixels, sizeof(unsigned char), tga->width * tga->height * tga->nchannels, file);
	fclose(file);
    // release the memory
	//free(imageData);

	return(TGA_OK);
}
/ *%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Ecriture d'un fichier TGA 32 bits (avec canal Alpha)
  if msk is null, the 4th channel of img is used.
  if used, msk has only one channel: this is the alpha.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_save_tga(PIX* img, PIX* msk, char* filename)
{
  if (msk == NULL && img->nchannels != 4)
  {
    printf("image does not have 4 channels plus no mask was provided, exit\n");
    return;
  }

  // open file for writing
  FILE *fp;
  fp = fopen(filename, "wb");
  if (fp == NULL) return;

  // garbage
		putc(0x00, fp);
		putc(0x00, fp);

  /* image type
		1	colour map image
		2	RGB(A) uncompressed
		3	greyscale uncompressed
		9	greyscale RLE (compressed)
		10	RGB(A) RLE (compressed) */
		putc(0x02, fp);

  // garbage
		putc(0x00, fp);
		putc(0x00, fp);
		putc(0x00, fp);
		putc(0x00, fp);
		putc(0x00, fp);
		putc(0x00, fp);
		putc(0x00, fp);
		putc(0x00, fp);
		putc(0x00, fp);

  // largeur de l'image
		putc((img->width & 0x00FF), fp);
		putc((img->width & 0xFF00) / 256, fp);

  // hauteur de l'image
		putc((img->height & 0x00FF), fp);
		putc((img->height & 0xFF00) / 256, fp);

  // pixel depth is 32 (4 channels X 8 bits per pixel) that's 0x20 in hexa
		putc(0x20, fp);//img->nchannels * 8, file);

  // garbage
		putc(0x08, fp);

  // Ecriture des données
  int x, y;

  if (msk != NULL) // separate mask was provided
  {
    //for(y=img->height-1; y>=0; y--)
    for(y=0; y<img->height; y++)
    {
      for(x=0; x<img->width; x++)
      {
        u8 *p = img->pixels + (x + y * img->width) *img->nchannels;
        u8 *m = msk->pixels + (x + y * msk->width);

        // Le format est BGRA (Blue Green Blue Alpha).
        putc(p[2], fp); // B
        putc(p[1], fp); // G
        putc(p[0], fp); // R
        putc(m[0], fp); // A is mask->GetRed(x, y)
      }
    }
  }
  else // PIX has 4 channels
  {
    //for(y=img->height-1; y>=0; y--)
    for(y=0; y<img->height; y++)
    {
      for(x=0; x<img->width; x++)
      {
        u8 *p = img->pixels + (x + y * img->width) *img->nchannels;
        // Le format est BGRA (Blue Green Blue Alpha)
        putc(p[2], fp); // B
        putc(p[1], fp); // G
        putc(p[0], fp); // R
        putc(p[3], fp); // A
      }
    }
  }
  // Fermeture du fichier
  fclose(fp);
}
