/*******************************************************************************
   ___ _      _ _ _     
  / _ (_)_  _| (_) |__  
 / /_)/ \ \/ / | | '_ \ 
/ ___/| |>  <| | | |_) |
\/    |_/_/\_\_|_|_.__/ 

 ** Copyright (C) 2008-2021 Masta 

 ** This software is provided 'as-is', without any express or implied
 ** warranty.  In no event will the authors be held liable for any damages
 ** arising from the use of this software.

 ******************************************************************************/
#include <pixlib.h>
#include <stdlib.h>
#include <string.h>

/*#ifdef WIN32
BOOL APIENTRY DllMain (HINSTANCE hInst, DWORD reason, LPVOID reserved )
{
  return TRUE;
}
#endif
#define PIXLIB_API _declspec (dllexport)
*/
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Créer un PIX vide (noir par défaut) aux dimensions x,y avec n channels
  La mémoire doit être libérée après utilisation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
PIX *pix_create(int width, int height, u8 nchannels)
{
  PIX *b = (PIX*) malloc(sizeof(PIX));
       b->width  = width;
       b->height = height;
       b->nchannels = nchannels;
       b->pixels = (u8 *)malloc(width*height*nchannels);
  return b;
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Obtenir les valeurs RGB d'un pixel en x,y
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_getPixel(PIX *img, int x, int y, RGBpixel *color)
{
	if((x > img->width) || (y > img->height)) return;
	u8 *p = img->pixels + (x + y * img->width) * img->nchannels;
	color->r = p[0];
	color->g = p[1];
	color->b = p[2];
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Obtenir les valeurs RGB d'un pixel en x,y
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
RGBpixel getPixel(PIX *img, int x, int y)
{
	u8 *p = img->pixels + (x + y * img->width) * img->nchannels;
	RGBpixel pixel = {p[0],p[1],p[2]};
	return pixel;
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Obtenir une composante RGB d'un pixel en x,y
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
u8 getRed (PIX *img, int x, int y){
	u8 *p = img->pixels + (x + y * img->width) * img->nchannels;
	return p[0];
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Obtenir une composante RGB d'un pixel en x,y
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
u8 getGreen (PIX *img, int x, int y){
	u8 *p = img->pixels + (x + y * img->width) * img->nchannels;
	return p[1];
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Obtenir une composante RGB d'un pixel en x,y
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
u8 getBlue (PIX *img, int x, int y){
	u8 *p = img->pixels + (x + y * img->width) * img->nchannels;
	return p[2];
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Définir les valeurs RGB d'un pixel en x,y
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_setPixel(PIX *img, int x, int y, u8 r, u8 g, u8 b)
{
	if((x > img->width) || (y > img->height)) return;
	u8 *p = img->pixels + (x + y * img->width) * img->nchannels;
	p[0] = r;
	p[1] = g;
	p[2] = b;
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Dupliquer un PIX. La mémoire doit être libérée après utilisation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
PIX *pix_copy (PIX *img)
{
	PIX *tmp = pix_create(img->width, img->height, img->nchannels);
	memcpy(tmp->pixels, img->pixels, img->width*img->height*img->nchannels);
  
	/* copie manuelle, plus lente
	for (int y=0; y<tmp->height; y++)
 		for (int x=0; x<tmp->width; x++)
		{
			int a = (x + y * tmp->width) * tmp->nchannels;
 			tmp->pixels[a+0] = img->pixels[a+0];
			tmp->pixels[a+1] = img->pixels[a+1];
			tmp->pixels[a+2] = img->pixels[a+2];
		}*/
	return tmp;
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Détruire un PIX: libère la mémoire
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pix_destroy(PIX *img)
{
	if(img->pixels) free(img->pixels);
	if(img) free(img); 
}

